<?php
/*
Plugin Name: GoTryTHIS
Plugin URI: http://www.gotrythis.com/help/tech-support/wordpress-org-plugin/
Description: The GoTryTHIS WordPress plugin redirects non-WordPress links through the GoTryTHIS link redirection application, thus eliminating conflicts between WordPress and GoTryTHIS.
Version: 0.3.2
Author: GoTryTHIS.com
Author URI: http://www.GoTryTHIS.com/
*/

$gotrythis_install_folder = '';

define('GOTRYTHIS_INSTALL_FOLDER', '%%gotrythis_install_folder%%');
define('GOTRYTHIS_DEFAULT_INSTALL_FOLDER', 'gotrythis');

function gotrythis_init() {
	global $gotrythis_install_folder;

	$gotrythis_install_folder = get_option('gotrythis_install_folder');

	if ($gotrythis_install_folder === false) {
		if (!preg_match('#^\%{2}(.*)\%{2}$#', GOTRYTHIS_INSTALL_FOLDER)) {
			update_option('gotrythis_install_folder', GOTRYTHIS_INSTALL_FOLDER);

			$gotrythis_install_folder = GOTRYTHIS_INSTALL_FOLDER;
		} else {
			update_option('gotrythis_install_folder', GOTRYTHIS_DEFAULT_INSTALL_FOLDER);

			$gotrythis_install_folder = GOTRYTHIS_DEFAULT_INSTALL_FOLDER;
		}
	}

	add_action('admin_menu', 'gotrythis_config_page');
	gotrythis_admin_warnings();
}

add_action('init', 'gotrythis_init');

if (!function_exists('wp_nonce_field')) {
	function gotrythis_nonce_field($action = -1) {
		return;
	}

	$gotrythis_nonce = -1;
} else {
	function gotrythis_nonce_field($action = -1) {
		return wp_nonce_field($action);
	}

	$gotrythis_nonce = 'gotrythis-update-install-folder';
}

function gotrythis_remove_slashes($string, $point) {
	if (empty($string)) {
		return '';
	}

	switch ($point) {
		case 'start' :
			$string = preg_replace('#^\/#', '', $string);

			break;

		case 'end' :
			$string = preg_replace('#\/$#', '', $string);

			break;

		case 'border' :
			$string = preg_replace('#^\/|\/$#', '', $string);

			break;

		default :
			$string = preg_replace('#\/{2,}#', '/', $string);
	}

	return $string;
}

function gotrythis_config_page() {
	if (function_exists('add_submenu_page')) {
		add_submenu_page('plugins.php', __('GoTryTHIS Configuration'), __('GoTryTHIS Configuration'), 'manage_options', 'gotrythis-install-folder-config', 'gotrythis_conf');
	}
}

function gotrythis_conf() {
	$ms = array();

	if (isset($_POST['submit'])) {
		if (function_exists('current_user_can') && !current_user_can('manage_options')) {
			die(__('Cheatin&#8217; uh?'));
		}

		check_admin_referer($gotrythis_nonce);

		$install_folder		   = trim($_POST['install_folder']);
		$install_folder_status = gotrythis_verify_install_folder($install_folder);

		if ($install_folder_status == 'valid') {
			update_option('gotrythis_install_folder', $install_folder);

			$ms[] = 'new_install_folder_valid';
		} elseif ($install_folder_status == 'invalid') {
			$ms[] = 'new_install_folder_invalid';
		} else if ($install_folder_status == 'failed') {
			$ms[] = 'new_install_folder_failed';
		}
	} else {
		$install_folder		   = gotrythis_get_install_folder();
		$install_folder_status = gotrythis_verify_install_folder($install_folder);

		if ($install_folder_status == 'invalid') {
			$ms[] = 'install_folder_invalid';
		} else if (($install_folder !== false) && ($install_folder_status == 'failed')) {
			$ms[] = 'install_folder_failed';
		}
	}

	$messages = array(
		'new_install_folder_valid'	 => array('color' => '2d2', 'text' => __('GoTryTHIS installation folder has been verified. You can use GoTryTHIS now.')),
		'new_install_folder_invalid' => array('color' => 'd22', 'text' => __('GoTryTHIS installation folder you entered is invalid. Please double-check it.')),
		'new_install_folder_failed'  => array('color' => 'd22', 'text' => __('GoTryTHIS installation folder cann\'t be verified because a connection to your server could not be established. Please check your server configuration.')),
		'install_folder_invalid'	 => array('color' => 'd22', 'text' => __('Current GoTryTHIS installation folder is invalid. Please check it.')),
		'install_folder_failed'		 => array('color' => 'd22', 'text' => __('GoTryTHIS installation folder cann\'t be verified because a connection to your server could not be established. Please check your server configuration.'))
	);
?>
<?php
	if (!empty($_POST['submit']) && ($install_folder_status == 'valid')) {
?>
<div id = "message" class = "updated fade"><p><strong><?php _e('Options saved.') ?></strong></p></div>
<?php
	}
?>
<div class="wrap">
	<h2><?php _e('GoTryTHIS Configuration'); ?></h2>
	<div class="narrow">
		<form action="" method="post" id="gotrythis-conf" style="margin: auto; width: 400px; ">
			<p><?php _e('The GoTryTHIS WordPress plugin redirects non-WordPress links through the GoTryTHIS link redirection application, thus eliminating conflicts between WordPress and GoTryTHIS.'); ?></p>
			<h3><label for="install_folder"><?php _e('GoTryTHIS Installation Folder'); ?></label></h3>
<?php
	foreach ($ms as $m) {
?>
			<p style="padding: .5em; background-color: #<?php echo $messages[$m]['color']; ?>; color: #fff; font-weight: bold;"><?php echo $messages[$m]['text']; ?></p>
<?php
	}
?>
			<p><input id="install_folder" name="install_folder" type="text" size="20" value="<?php echo $install_folder; ?>" style="font-family: 'Courier New', Courier, mono; font-size: 1.5em;" /> (<?php _e('<a href="http://help.gotrythis.com/entries/469854-wordpress-compatibility">What is this?</a>'); ?>)</p>
<?php
	if ($install_folder_status == 'invalid') {
?>
			<ol type = "1">
				<li><?php _e('Does the folder name match where you installed GoTryTHIS?'); ?></li>
				<li><?php _e('Check the format: Example, "gotrythis" or "/gotrythis" is correct and "\gotrythis" is not.'); ?></li>
		 		<li><?php _e('Are there invalid characters like spaces?'); ?></li>
		 		<li><?php _e('Is the path relative to the home page directory?'); ?></li>
		 		<li><?php _e('If your links are working and your blog is in maintenance mode, ignore this error.'); ?></li>
		 	</ol>
<?php
	}
?>
<?php
	gotrythis_nonce_field($gotrythis_nonce);
?>
			<p class="submit"><input type="submit" name="submit" value="<?php _e('Update options &raquo;'); ?>" /></p>
		</form>
	</div>
</div>
<?php
}

function gotrythis_get_install_folder() {
	global $gotrythis_install_folder;

	if ($gotrythis_install_folder !== false) {
		return $gotrythis_install_folder;
	}

	return get_option('gotrythis_install_folder');
}

function gotrythis_verify_install_folder($install_folder) {
	$install_folder = gotrythis_remove_slashes($install_folder, 'extra');

	if (!preg_match('#^/?((\w+\.?)*\w+/?)*$#', $install_folder)) {
		return 'invalid';
	}

	$gotrythis_install_folder = gotrythis_get_install_folder();
	$blog_url_details		  = parse_url(get_option('siteurl'));
	$path					  = '/iqurguhfsdfglkj123';

	update_option('gotrythis_install_folder', $install_folder);

	$test1_response = gotrythis_http_get('gotrythis-redirect-test=1', $blog_url_details['host'], $path);
	$test2_response = gotrythis_http_get('redirect-test=1', $blog_url_details['host'], $path);

	update_option('gotrythis_install_folder', $gotrythis_install_folder);

	if (!is_array($test1_response) || !isset($test1_response[1])) {
		if (!is_array($test2_response) || !isset($test2_response[1])) {
			return 'failed';
		}
	}

	if (!preg_match('#gotrythis-redirect-test#', trim((string)$test1_response[1]))) {
		if (!preg_match('#redirect-test#', trim((string)$test2_response[1]))) {
			return 'invalid';
		}
	}

	return 'valid';
}

function gotrythis_http_get($request, $http_host, $path) {
	$http_request  = 'GET ' . $path . (!empty($request)? '?' . $request : '') . " HTTP/1.0\r\n";
	$http_request .= 'HOST: ' . $http_host . "\r\n";
	$http_request .= 'Connection: Close' . "\r\n\r\n";

	$response = '';

	if (($fs = @fsockopen($http_host, 80, $errno, $errstr, 10)) != false) {
		fwrite($fs, $http_request);

		while (!feof($fs)) {
			$response .= fread($fs, 1024); // One TCP-IP packet
		}

		fclose($fs);

		$response = explode("\r\n\r\n", $response, 2);
	}

	return $response;
}

function gotrythis_admin_warnings() {
	if (empty($_POST['submit'])) {
		function gotrythis_check_redirect() {
			$install_folder = gotrythis_get_install_folder();

			if (gotrythis_verify_install_folder($install_folder) != 'valid') {
				echo '<div id = "gotrythis-warning" class = "updated fade"><p><strong>' . __('GoTryTHIS plugin is configured incorrectly.') . '</strong> ' . sprintf(__('You have to <a href="%1$s">set correct GoTryTHIS installation folder</a> for it to work.'), 'plugins.php?page=gotrythis-install-folder-config') . '</p></div>';
			}
		}

		add_action('admin_notices', 'gotrythis_check_redirect');

		return;
	}
}

if (!function_exists('strhex')) {
	function strhex($str) {
		if (empty($str)) {
			return '';
		}

		$str_length = strlen($str);
		$strhex		= '';

		for ($i = 0; $i < $str_length; $i++) {
			$strhex .= substr('00' . strtoupper(dechex(ord($str{$i}))), -2);
		}

		return $strhex;
	}
}

function gotrythis_cookie_test() {
	if (!empty($_SERVER['REQUEST_URI'])) {
		$url = $_SERVER['REQUEST_URI'];
	} elseif (!empty($_SERVER['HTTP_X_REWRITE_URL'])) {
    	$url = $_SERVER['HTTP_X_REWRITE_URL'];
	} elseif (!empty($_SERVER['QUERY_STRING'])) {
    	$pattern = '`.+' . $_SERVER['SERVER_NAME'] . '(:[0-9]+)?(/.+)`';

	    if (preg_match_all($pattern, $_SERVER['QUERY_STRING'], $url)) {
	    	$url = $url[2][0];
	    } else {
	    	$url = null;
	    }
	}

	if (!empty($url)) {
		if (preg_match('`(.+)\?(.+)`', $url, $res)) {
		    $code = $res[1];
		} else {
			$code = $url;
		}

		if (!empty($code) && $code{0} == '/') {
	    	$code = substr($code, 1);
		}

		if (!empty($code) && substr($code, -1) == '/') {
	    	$code = substr($code, 0, -1);
		}
	} else {
		$code = '';
	}

	$site_url_details = parse_url(strtolower(get_option('siteurl')));
	$dom_			  = str_replace('.', '_', preg_replace('#^www\.#', '', $site_url_details['host']));

	if (!empty($_COOKIE['gtt_redirect' . md5($dom_ . '_' . $code)]) || !empty($_COOKIE[strhex('gotrythis_redirect_' . $dom_ . '_' . $code)])) {
		return true;
	}

	return false;
}

function get_gotrythis_redirect_path() {
	$gotrythis_install_folder = gotrythis_remove_slashes(gotrythis_get_install_folder(), 'border');
	$gotrythis_redirect_path  = $gotrythis_install_folder . '/gotrythis.php';
	$site_url_details		  = parse_url(get_option('siteurl'));
	$wp_install_path		  = gotrythis_remove_slashes($site_url_details['path'], 'border');

	if (!empty($wp_install_path)) {
		$gotrythis_redirect_path = str_repeat('../', count(explode('/', $wp_install_path))) . $gotrythis_redirect_path;
	}

	return $gotrythis_redirect_path;
}

function check_gotrythis_redirect() {
	if (isset($_GET['gotrythis-plugin-redirect-test'])) {
		echo 'gotrythis-plugin-redirect-test';

		exit();
	}

	if (isset($_GET['gotrythis-plugin-get-gotrythis-install-folder'])) {
		echo gotrythis_get_install_folder();

		exit();
	}

	$gtt_redirect_path = get_gotrythis_redirect_path();

	if (!is_404() || !file_exists($gtt_redirect_path) || gotrythis_cookie_test()) {
		return false;
	}

	return true;
}

function gotrythis_redirect() {
	$gtt_redirect_path = get_gotrythis_redirect_path();

	if (check_gotrythis_redirect()) {
		if (!defined('DONOTCACHEPAGE')) {
			define('DONOTCACHEPAGE', true);
		}

		if (!defined('OUT_SOFT_REDIRECT')) {
			define('OUT_SOFT_REDIRECT', true);
		}

		if (!defined('GTT_OUT_SOFT_REDIRECT')) {
			define('GTT_OUT_SOFT_REDIRECT', true);
		}

		chdir(dirname($gtt_redirect_path));

		require_once basename($gtt_redirect_path);

		exit();
	}
}

add_action('template_redirect', 'gotrythis_redirect', 1);

?>